@@ -4,22 +4,20 @@
 # @Email  : 877362867@qq.com
 # @Date   : 2021/08/31 09:56
 
+from pyxllib.prog.pupil import check_install_package
+
+check_install_package('win32com', 'pypiwin32')
+
 import json
 import os
 import re
-import subprocess
-
-try:
-    import win32com
-except ModuleNotFoundError:
-    subprocess.run(['pip', 'install', 'pypiwin32'])
 
 import pythoncom
 from win32com.client import constants
 import win32com.client as win32
 
 from pyxllib.prog.newbie import RunOnlyOnce
-from pyxllib.prog.pupil import DictTool, EnchantBase
+from pyxllib.prog.pupil import DictTool, EnchantBase, EnchantCvt
 from pyxllib.text.pupil import strwidth
 from pyxllib.debug.specialist import File, Dir, get_etag, browser
 
@@ -33,11 +31,8 @@ def __docx():
 class DocxTools:
     @classmethod
     def to_pdf(cls, docx_file, pdf_file=None):
-        try:
-            import docx2pdf  # pip install docx2pdf，并且本地要安装word软件
-        except ModuleNotFoundError:
-            subprocess.run('pip3 install docx2pdf')  # 安装不成功的时候可以考虑加参数：--user
-            import docx2pdf
+        check_install_package('docx2pdf')  # 安装不成功的时候可以考虑加参数：--user
+        import docx2pdf
 
         if pdf_file is None:
             pdf_file = docx_file.with_suffix('.pdf')
@@ -46,15 +41,15 @@ class DocxTools:
         return pdf_file
 
     @classmethod
-    def merge(cls, master_file, toc, *, subtoc='reduce'):
+    def merge(cls, master_file, toc, *, outline='demote'):
         """ 合并多份docx文件
 
         :param master_file: 要合并到哪个主文件
             注意如果这个文件已存在，会被替换，重置
         :param toc: 类似fitz的table of contents，用一个n*3的表格表示新文档的格式
             list，每个元素三列：标题级别，标题名称，(可选)对应文件内容
-        :param subtoc: 原来每份子文档里的标题，插入到新文档中的处理规则
-            reduce：降级
+        :param outline: 原来每份子文档里的标题，插入到新文档中的处理规则
+            demote：降级
             clear：清除
 
         这个功能还有些局限性，后面要扩展鲁棒性
@@ -75,9 +70,15 @@ class DocxTools:
             if file:
                 file = File(file)
                 member_doc = app.open_doc(file)
-
                 member_doc.Activate()
 
+                # 处理原来文档的目录级别
+                if outline == 'demote':
+                    member_doc.outline_demote(lvl)
+                elif outline == 'clear':
+                    # 降10级就相当于清除所有原来的标题
+                    member_doc.outline_demote(10)
+
                 app.Selection.WholeStory()
                 app.Selection.Copy()
                 master_doc.Activate()
@@ -85,7 +86,7 @@ class DocxTools:
                 app.Selection.Paste()
                 member_doc.Close()
         master_doc.save()
-        master_doc.Close()
+        master_doc.Close(True)
 
 
 class Document:
@@ -162,8 +163,6 @@ class Document:
 
         本质是把docx转成pdf，利用pdf的解析生成labelme格式的标准框查看
 
-        :param docx_file: 注意写的是docx文件的路径，然后pdf、png、json都会放在同目录下
-            这个不适合设成可选参数，需要显式指定一个输出目录比较好
         :param views: 详见to_labelmes的描述
             各位依次代表是否显示对应细粒度的标注：blocks、lines、spans、chars
         :param bool|dict advance: 是否开启“高级”功能，开启后能获得下划线等属性，但速度会慢很多
@@ -276,9 +275,9 @@ class EnchantWin32WordApplication(EnchantBase):
         """
         # app
         _cls = type(app)
-        names = cls.check_enchant_names([_cls])
+        names = cls.check_enchant_names([_cls], ignore_case=True)
         exclude_names = {'get_app'}
-        cls._enchant(_cls, names - exclude_names, mode='staticmethod2objectmethod')
+        cls._enchant(_cls, names - exclude_names)
 
         if recursion_enchant:
             # 建一个临时文件，把各种需要绑定的对象都生成绑定一遍
@@ -297,35 +296,31 @@ class EnchantWin32WordApplication(EnchantBase):
             doc.Close(False)
 
     @classmethod
-    def get_app(cls, mode='default', *, visible=None, display_alerts=0, recursion_enchant=True):
+    def get_app(cls, app=None, *, visible=None, display_alerts=0, recursion_enchant=True):
         """
         Args:
-            mode: 目前除了默认default，只有new，强制新建一个app
-                210912周日10:07，可以先不管这个参数，使用默认模式就好了，现在所有app能统一为一个了
-                    之前是好像app会有多个版本，导致Documents管理不集中，但同名文件又会夸app冲突很麻烦
-                    所以才搞的这个复杂机制。如果该问题暂未出现，那么mode其实没用，多此一举。
+            app: 可以自定义在外部使用Dispatch、DispatchEx等形式给入已初始化好的app
             visible: 是否可见
             display_alerts: 是否关闭警告
             recursion_enchant: 是否递归执行enchant
-
         """
-        if mode == 'Dispatch':
-            app = win32.Dispatch('WORD.APPLICATION')
-        elif mode == 'DispatchEx':
-            # 遇过win7+word2013，只能用DispatchEx的情况
-            app = win32.DispatchEx('WORD.APPLICATION')
-        elif mode == 'EnsureDispatch':
-            app = win32.gencache.EnsureDispatch('WORD.APPLICATION')
-        else:
+        # 1 get app
+        name = 'WORD.APPLICATION'
+        if app is None:
+            try:
+                app = win32.GetActiveObject(name)
+            except pythoncom.com_error:
+                pass
+        if app is None:
             try:
                 # 名称用大写，会比较兼容旧的word2013等版本
-                app = win32.GetActiveObject('WORD.APPLICATION')
-            except pythoncom.com_error:
-                app = None
-        if app is None:
-            # 必须用gencache方法，才能获得 from win32com.client import constants 的常量
-            app = win32.gencache.EnsureDispatch('WORD.APPLICATION')
-            # print('gencache')
+                # 尽量静态调度，才能获得 from win32com.client import constants 的常量
+                app = win32.gencache.EnsureDispatch(name)
+            except TypeError:
+                # 实在不行，就用动态调度
+                app = win32.dynamic.Dispatch(name)
+
+        # 2 enchant
         cls.enchant(app, recursion_enchant=recursion_enchant)
 
         if visible is not None:
@@ -390,10 +385,10 @@ class EnchantWin32WordDocument(EnchantBase):
     @RunOnlyOnce.decorator(distinct_args=False)
     def enchant(cls, doc):
         _cls = type(doc)
-        names = cls.check_enchant_names([_cls])
-        propertys = {'n_page', 'content'}
-        cls._enchant(_cls, propertys, mode='staticmethod2property')
-        cls._enchant(_cls, names - propertys, mode='staticmethod2objectmethod')
+        names = cls.check_enchant_names([_cls], ignore_case=True)
+        propertys = {'n_page'}
+        cls._enchant(_cls, propertys, EnchantCvt.staticmethod2property)
+        cls._enchant(_cls, names - propertys)
 
     @staticmethod
     def save(doc, file_name=None, fmt=None, retain=False, **kwargs):
@@ -421,13 +416,17 @@ class EnchantWin32WordDocument(EnchantBase):
             10
             """
             # 复杂格式可能无法完美支持所有功能。比如复杂的pdf无法使用SaveAs2实现，要用ExportAsFixedFormat。
-            common = {'doc': 'FormatDocument97',
-                      'html': 'FormatHTML',
-                      'txt': 'FormatText',
-                      'docx': 'FormatDocumentDefault',
-                      'pdf': 'FormatPDF'}
+            # 有些情况可能无法使用gencache，导致没有constants，所以默认可以直接映射到整数，避免使用到constants
+            common = {'doc': 0,
+                      'html': 8,
+                      'txt': 2,
+                      'docx': 16,
+                      'pdf': 17}
             name = common.get(fmt.lower().lstrip('.'), fmt)
-            return getattr(constants, 'wd' + name)
+            if isinstance(name, int):
+                return name
+            else:
+                return getattr(constants, 'wd' + name)
 
         # 2 确认要存储的文件格式
         if isinstance(fmt, str):
@@ -455,7 +454,7 @@ class EnchantWin32WordDocument(EnchantBase):
                 outfile = File(doc.Name, doc.Path, suffix='.' + fmt)
                 doc.SaveAs2(str(outfile), save_format(outfile.suffix), **kwargs)
             else:
-                etag = get_etag(doc.content)
+                etag = get_etag(doc.Content)
                 outfile = File(etag, Dir.TEMP, suffix=fmt)
                 doc.SaveAs2(str(outfile), save_format(fmt), **kwargs)
 
@@ -472,9 +471,10 @@ class EnchantWin32WordDocument(EnchantBase):
         else:
             return outfile
 
-    @staticmethod
-    def content(doc):
-        return doc.Range().content
+    # 先暂时不开启 doc.chars
+    # @staticmethod
+    # def chars(doc):
+    #     return doc.Range().chars
 
     @staticmethod
     def n_page(doc):
@@ -522,12 +522,32 @@ class EnchantWin32WordDocument(EnchantBase):
                     doc.Range(r.Start, r.End - 1).InsertAfter(f'，{size}')
 
     @staticmethod
-    def outline_demote(doc, level):
+    def outline_demote(doc, demote_level):
         """ 标题降级，降低level层 """
         for p in doc.Paragraphs:
-            name = p.Range.Style.NameLocal
-            m = re.match(r'标题 (\d)$', name)
-            lvl = int(m.groups(1))
+            p.Range.demote(demote_level)
+
+    @staticmethod
+    def set_style(doc, obj, name):
+        """ 给Paragraph、Range等按名称设置样式
+
+        :param obj: 当前doc下某个含有Style成员属性的子对象
+        :param name: 样式名称
+        """
+        setattr(obj, 'Style', doc.Styles(name))
+
+    @staticmethod
+    def add_paragraph(doc, text='', style=None):
+        """ 自定义的插入段落
+
+        默认的插入比较麻烦，新建段落、插入文本、设置格式要多步实现，这里封装支持在一步进行多种操作
+        """
+        p = doc.Paragraphs.Add()
+        if text:
+            p.Range.InsertBefore(text)
+        if style:
+            p.Style = doc.Styles(style)
+        return p
 
 
 class EnchantWin32WordRange(EnchantBase):
@@ -541,10 +561,10 @@ class EnchantWin32WordRange(EnchantBase):
     @RunOnlyOnce.decorator(distinct_args=False)
     def enchant(cls, rng):
         _cls = type(rng)
-        names = cls.check_enchant_names([_cls])
-        propertys = {'content'}
-        cls._enchant(_cls, propertys, mode='staticmethod2property')
-        cls._enchant(_cls, names - propertys, mode='staticmethod2objectmethod')
+        names = cls.check_enchant_names([_cls], ignore_case=True)
+        propertys = {'chars'}
+        cls._enchant(_cls, propertys, EnchantCvt.staticmethod2property)
+        cls._enchant(_cls, names - propertys)
 
     @staticmethod
     def set_hyperlink(rng, url):
@@ -554,7 +574,7 @@ class EnchantWin32WordRange(EnchantBase):
         doc.Hyperlinks.Add(rng, url)
 
     @staticmethod
-    def content(rng):
+    def chars(rng):
         # 有特殊换行，ch.Text可能会得到 '\r\x07'，为了位置对应，只记录一个字符
         return ''.join([ch.Text[0] for ch in rng.Characters])
 
@@ -574,16 +594,93 @@ class EnchantWin32WordRange(EnchantBase):
         start_idx, end_idx = rng.Characters(start + 1).Start, rng.Characters(end).End
         return rng.Document.Range(start_idx, end_idx)
 
+    @staticmethod
+    def shifting(rng, left=0, right=0):
+        """ range左右两边增加偏移量，返回重定位的rng
+
+        常用语段落定位，要在段落末尾增加内容时
+        >> rng2 = p.Range.shifting(right=-1)
+        """
+        return rng.Document.Range(rng.Start + left, rng.End + right)
+
+    @staticmethod
+    def demote(rng, demote_level):
+        """ 标题降级，降低level层 """
+        name = rng.Style.NameLocal  # 获得样式名称
+        m = re.match(r'标题 (\d)$', name)
+        if m:
+            lvl = int(m.group(1))
+            new_lvl = lvl + demote_level
+            new_style = f'标题 {new_lvl}' if new_lvl < 10 else '正文'
+            rng.Style = rng.Parent.Styles(new_style)
+
+    @staticmethod
+    def set_font(rng, font_fmt):
+        """ 设置字体各种格式
+
+        :param dict|str font_fmt:
+            dict，定制格式
+                布尔类型：Bold、Italic、Subscript、Superscript
+                可布尔的值类型：Underline
+                    支持的格式见：https://docs.microsoft.com/en-us/office/vba/api/word.wdunderline
+                值类型：Name、Size、Color、UnderlineColor
+            str，使用现有样式名
+        """
+        if isinstance(font_fmt, dict):
+            font = rng.Font
+            for k, v in font_fmt.items():
+                setattr(font, k, v)
+        elif isinstance(font_fmt, str):
+            rng.Style = rng.Parent.Styles(font_fmt)
+        else:
+            raise ValueError
+
+    @staticmethod
+    def insert_before(rng, text, font_fmt=None):
+        """ 对原InsertBefore的功能封装
+
+        :return: 增加返回值，是新插入内容的range定位
+        """
+        start1, end1 = rng.Start, rng.End
+        rng.InsertBefore(text)
+        bias = rng.End - end1  # 新插入的内容大小
+        new_rng = rng.Document.Range(start1, start1 + bias)
+        if font_fmt:
+            new_rng.set_font(font_fmt)
+        return new_rng
+
+    @staticmethod
+    def insert_after(rng, text, font_fmt=None):
+        """ 同insert_before，是InsertAfter的重封装
+        """
+        # 1
+        start1, end1 = rng.Start, rng.End
+
+        # 2 往后插入，会排除\r情况
+        doc = rng.Document
+        ch = doc.Range(end1 - 1, end1).Text
+        if ch == '\r':
+            end1 -= 1
+            rng = doc.Range(start1, end1)
+
+        # 3
+        rng.InsertAfter(text)
+        bias = rng.End - end1
+        new_rng = rng.Document.Range(end1, end1 + bias)
+        if font_fmt:
+            new_rng.set_font(font_fmt)
+        return new_rng
+
 
 class EnchantWin32WordHyperlink(EnchantBase):
     @classmethod
     @RunOnlyOnce.decorator(distinct_args=False)
     def enchant(cls, link):
         _cls = type(link)
-        names = cls.check_enchant_names([_cls])
+        names = cls.check_enchant_names([_cls], ignore_case=True)
         propertys = {'netloc', 'name'}
-        cls._enchant(_cls, propertys, mode='staticmethod2property')
-        cls._enchant(_cls, names - propertys, mode='staticmethod2objectmethod')
+        cls._enchant(_cls, propertys, EnchantCvt.staticmethod2property)
+        cls._enchant(_cls, names - propertys)
 
     @staticmethod
     def netloc(link):
